/*************************************************************************
 *                                                                       *
 *                              MEMFILE.CPP                              *
 *                                                                       *
 *************************************************************************

                          Copyright 1996 Echidna

   DESCRIPTION


   PROGRAMMERS


   FUNCTIONS

   TABS : 5 9

   HISTORY
		03/28/96 : Created.


	The Echidna Copyright
	
	Copyright 1991-2003 Echidna, Inc. All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are
	met:
	
	* Redistributions of source code must retain the above copyright notice,
	  this list of conditions and the following disclaimer.
	
	* Redistributions in binary form must reproduce the above copyright
	  notice, this list of conditions and the following disclaimer in the
	  documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED BY Echidna ``AS IS'' AND ANY EXPRESS OR IMPLIED
	WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
	MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
	NO EVENT SHALL Echidna OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
	
	The views and conclusions contained in the software and documentation are
	those of the authors and should not be interpreted as representing
	official policies, either expressed or implied, of Echidna or
	Echidna, Inc.


 *************************************************************************/

/**************************** I N C L U D E S ****************************/
#include "stdafx.h"
#include "stdio.h"
#include "memfile.h"


/*************************** C O N S T A N T S ***************************/


/******************************* T Y P E S *******************************/


/************************** P R O T O T Y P E S **************************/


/***************************** G L O B A L S *****************************/


/****************************** M A C R O S ******************************/


/**************************** R O U T I N E S ****************************/

MEMFILE *MEMFILE_Load (const char *filename)
{
	MEMFILE* mf = NULL;
	HANDLE	fh;
	DWORD	len;
	
	fh = CreateFile (filename, GENERIC_READ, FILE_SHARE_READ|FILE_SHARE_READ|FILE_SHARE_DELETE, NULL, OPEN_EXISTING, 0, NULL);
	if (fh != INVALID_HANDLE_VALUE)
	{
		len = GetFileSize(fh, NULL);
		if (len && len != INVALID_FILE_SIZE)
		{
			mf = (MEMFILE*)HeapAlloc (GetProcessHeap(), 0 ,len  + sizeof (MEMFILE));
			if (mf)
			{
                DWORD numRead;

				FillMemory (mf, sizeof (MEMFILE), 0);

				mf->buffer = ((uint8*)mf)+sizeof (MEMFILE);
			
				mf->curPtr    = mf->buffer;
				mf->size      = len;
				mf->bytesLeft = len;
	
				ReadFile (fh, mf->buffer, len, &numRead, NULL);
			}
		}
		CloseHandle (fh);
	}
	return mf;
}

void MEMFILE_Close (MEMFILE *mf)
{
	HeapFree (GetProcessHeap(), 0, mf);
}

int MEMFILE_Read(MEMFILE *mf, void *buf, long len)
{
	if (mf->bytesLeft)
	{
		long	copyBytes;

		copyBytes = (mf->bytesLeft < len) ? mf->bytesLeft : len;

		CopyMemory (buf, mf->curPtr, copyBytes);

		mf->bytesLeft -= copyBytes;
		mf->curPtr    += copyBytes;

		return copyBytes;
	}
	return 0;
}

int MEMFILE_Seek (MEMFILE *mf, long pos, int type)
{
	switch (type)
	{
	case SEEK_SET:
		mf->curPtr    = mf->buffer + pos;
		mf->bytesLeft = mf->size   - pos;
		break;
	case SEEK_CUR:
		mf->curPtr    += pos;
		mf->bytesLeft -= pos;
		break;
	case SEEK_END:
		mf->curPtr     = mf->buffer + mf->size + pos;
		mf->bytesLeft  = -pos;
		break;
	}
	return mf->size - mf->bytesLeft;
}


